#-*- coding:utf-8 -*-

#    Gedit a_href_picture plugin version 3.12.0
#    Copyright © 2011-2013 sasakima-nao <sasakimanao@gmail.com>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    2014.11.17 Version 3.12.0
#    Support Gedit 3.12
#    Keyboard ShortCut <Control><Alt>P
#
#    2014.01.03 Version 3.1.1
#    if not the Image then warning dialog
#    Under 300x300px is Not create subpicture
#    Fixed: Division '/' to '//'
#
#    2013.11.03 Version 3.1.0
#    Support Gedit 3.8 (Python3)
#
#    2011.07.18 Version 3.0.0

from gi.repository import GObject, Gedit, Gtk, Gio, GdkPixbuf
import os

MAX_SIZE = 300

class AHrefPictureAppActivatable(GObject.Object, Gedit.AppActivatable):
    """
        Set GMenu and Accelerator
    """
    app = GObject.property(type=Gedit.App)
 
    def __init__(self):
        GObject.Object.__init__(self)
 
    def do_activate(self):
        self.app.add_accelerator("<Control><Alt>P", "win.a_href_picture", None)
        self.menu_ext = self.extend_menu("file-section")
        item = Gio.MenuItem.new("a href Picture",  "win.a_href_picture")
        self.menu_ext.append_menu_item(item)
 
    def do_deactivate(self):
        self.app.remove_accelerator("win.a_href_picture", None)

class AHrefPicture(GObject.Object, Gedit.WindowActivatable):
    """
        Create small image and link insert
        (e.g. <a href="img/pic1.jpg"><img src="img/pic1-300x225.jpg" alt="img/pic1.jpg" /></a>)
        This Plugin Gedit 3 only
    """
    __gtype_name__ = "AHrefPicture"
    window = GObject.property(type=Gedit.Window)
    def __init__(self):
        GObject.Object.__init__(self)

    def do_activate(self):
        self.action = Gio.SimpleAction.new("a_href_picture", None)
        self.action.connect('activate', self.on_a_href_activate)
        self.window.add_action(self.action)

    def do_deactivate(self):
        self.window.remove_action("a_href_picture")

    def do_update_state(self):
        self.action.set_enabled(self.window.get_active_document() != None)

    def on_a_href_activate(self, action, data=None):
        doc = self.window.get_active_document()
        viewpath = doc.get_uri_for_display()
        if viewpath[0] != "/":
            self.messagebox("Please save the file")
            return
        dlg = Gtk.FileChooserDialog(
                "Select Picture",
                self.window,
                Gtk.FileChooserAction.OPEN,
                (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL, Gtk.STOCK_OPEN, Gtk.ResponseType.OK) )
        dlg.set_current_folder(os.path.dirname(viewpath))
        r = dlg.run()
        if r == Gtk.ResponseType.OK:
            try:
                filepath = dlg.get_filename()
                pixbuf = GdkPixbuf.Pixbuf.new_from_file(filepath)
            except:
                dlg.destroy()
                self.messagebox("This File is Not Picure!")
                return
            # create under 300*300 px image
            d_width = MAX_SIZE
            d_height = MAX_SIZE
            p_width = pixbuf.get_width()
            p_height = pixbuf.get_height()
            width = p_width
            height = p_height
            cleate = False
            smallpath = ""
            if width > MAX_SIZE or height > MAX_SIZE:
                cleate = True
                if (d_width * p_height) > (d_height * p_width):
                    width = p_width * d_height // p_height
                    height = d_height
                else:
                    width = d_width
                    height = p_height * d_width // p_width
                smallpix = GdkPixbuf.Pixbuf.scale_simple(pixbuf, width, height, GdkPixbuf.InterpType.BILINEAR)
                # jpeg or png
                name, ext = os.path.splitext(filepath)
                ext = ext.lower()
                if ext == ".jpg" or ext == ".jpeg":
                    smallpath = "{0}-{1}x{2}.jpg".format(name, width, height)
                    smallpix.savev(smallpath, "jpeg", ["quality"], ["85"])
                elif ext == ".png":
                    smallpath = "{0}-{1}x{2}.png".format(name, width, height)
                    smallpix.savev(smallpath, "png", ["compression"], ["9"])
                else:
                    self.messagebox("Extension not found")
                    return
            # create image link text
            dpath = os.path.dirname(viewpath)
            path1 = os.path.relpath(filepath, dpath)
            if cleate:
                path2 = os.path.relpath(smallpath, dpath)
                result = '<a href="{0}"><img src="{1}" alt="{0}" /></a>'.format(path1, path2)
            else:
                result = '<a href="{0}"><img src="{0}" alt="{0}" /></a>'.format(path1)
            # insert text
            view = self.window.get_active_view()
            buf = view.get_buffer()
            buf.insert_at_cursor(result)
        dlg.destroy()

    def messagebox(self, text):
        dlg = Gtk.MessageDialog(
                self.window,
                Gtk.DialogFlags.MODAL,
                Gtk.MessageType.WARNING,
                Gtk.ButtonsType.OK,
                text)
        r = dlg.run()
        dlg.destroy()
